/*
* Copyright 2021 VIA Technologies, Inc. All Rights Reserved.
*
* This PROPRIETARY SOFTWARE is the property of VIA Technologies, Inc.
* and may contain trade secrets and/or other confidential information of
* VIA Technologies, Inc. This file shall not be disclosed to any third
* party, in whole or in part, without prior written consent of VIA.
*
* THIS PROPRIETARY SOFTWARE AND ANY RELATED DOCUMENTATION ARE PROVIDED AS IS,
* WITH ALL FAULTS, AND WITHOUT WARRANTY OF ANY KIND EITHER EXPRESS OR IMPLIED,
* AND VIA TECHNOLOGIES, INC. DISCLAIMS ALL EXPRESS OR IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
* ENJOYMENT OR NON-INFRINGEMENT.
*/
#ifndef HARPCAM_H
#define HARPCAM_H

#include <Arduino.h>
#include <ArduinoJson.h>


#define HARPCAM_VERSION 0.1.0
#define MAX_OBJS		  3

#define PWDN_GPIO_NUM	  32
#define RESET_GPIO_NUM	  -1
#define XCLK_GPIO_NUM	   0
#define SIOD_GPIO_NUM	  26
#define SIOC_GPIO_NUM	  27

#define Y9_GPIO_NUM		  35
#define Y8_GPIO_NUM		  34
#define Y7_GPIO_NUM		  39
#define Y6_GPIO_NUM		  36
#define Y5_GPIO_NUM		  21
#define Y4_GPIO_NUM		  19
#define Y3_GPIO_NUM		  18
#define Y2_GPIO_NUM		   5
#define VSYNC_GPIO_NUM	  25
#define HREF_GPIO_NUM	  23
#define PCLK_GPIO_NUM	  22


class Harpcam
{
public:
	Harpcam();
	
	enum EFunc
	{
		FUNC_COLOR_DETECTION		= 1,
		FUNC_COLOR_CODE_DETECTION	= 2,
		FUNC_SHAPE_DETECTION		= 3,
		FUNC_SPHERE_DETECTION		= 4,
		FUNC_TEMPLATE_MATCHING		= 5,
		FUNC_KEYPOINTS				= 8,
		FUNC_NEURAL_NETWORK			= 9,
		FUNC_APRILTAG				= 10,
		FUNC_TRAFFIC_SIGN_DETECTION	= 12,
		FUNC_HANDWRITTEN_DIGITS_DETECTION	= 13,
		FUNC_HANDWRITTEN_LETTERS_DETECTION	= 14,
		FUNC_EQUATION_DETECTION		= 17,
		FUNC_LANES_DETECTION		= 16,
		FUNC_FACE_DETECTION			= 31,
	};

	enum EColorID
	{
		ID_RED = 1,
		ID_YELLOW,
		ID_GREEN,
		ID_BLUE,
		ID_PURPLE,
		ID_BLACK
	};

	enum EColor
	{
		COLOR_RED = 0x1,
		COLOR_YELLOW = 0x2,
		COLOR_GREEN = 0x4,
		COLOR_BLUE = 0x8,
		COLOR_PURPLE = 0x10,
		COLOR_BLACK = 0x20
	};

	enum EShapeID
	{
		ID_ROUND = 1,
		ID_RECTANGLE,
		ID_TRIANGLE,
		ID_PENTAGON
	};

	enum EShape
	{
		SHAPE_ROUND = 0x1,
		SHAPE_RECTANGLE = 0x2,
		SHAPE_TRIANGLE = 0x4,
		SHAPE_PENTAGON = 0x8
	};

	enum ETrafficSign
	{
		SIGN_NO_ENTRE = 0,
		SIGN_NO_LEFT_TURN,
		SIGN_NO_RIGHT_TURN,
		SIGN_WRONG_WAY,
		SIGN_NO_U_TURN,
		SIGN_MAX_SPEED,
		SIGN_ONEWAY_TRAFFIC,
		SIGN_LEFT_TURN,
		SIGN_RIGHT_TURN,
		SIGN_MIN_SPEED,
		SIGN_U_TURN,
		SIGN_TUNNEL_AHEAD,
		SIGN_BEWARE_OF_CHILDREN,
		SIGN_ROUNDABOUT,
		SIGN_YIELD_TO_PEDESTRIAN,
		SIGN_RED_LIGHT,
		SIGN_GREEN_LIGHT
	};

	enum ELetters
	{
		LETTER_A=0,
		LETTER_B,
		LETTER_C,
		LETTER_D,
		LETTER_E,
		LETTER_F,
		LETTER_G,
		LETTER_H,
		LETTER_I,
		LETTER_J,
		LETTER_K,
		LETTER_L,
		LETTER_M,
		LETTER_N,
		LETTER_O,
		LETTER_P,
		LETTER_Q,
		LETTER_R,
		LETTER_S,
		LETTER_T,
		LETTER_U,
		LETTER_V,
		LETTER_W,
		LETTER_X,
		LETTER_Y,
		LETTER_Z
	};

	enum EState
	{
		CMD = 1,
		IMG,
	};

	struct OutputData {
		int type;
		int x;
		int y;
		int w;
		int h;
		float confidence;
	};
	
	struct ApriltagExtraData {
		int posx;
		int posy;
		int posz;
		int rotx;
		int roty;
		int rotz;
		int centerx;
		int centery;
	};

	struct LanesExtraData {
		int centerx;
		int centery;
		int lx1;
		int ly1;
		int lx2;
		int ly2;
		int rx1;
		int ry1;
		int rx2;
		int ry2;
	};

	bool setupCamera(); //setup ov2640 on ESP32
	bool isDetected();
	bool parseJsonContent(String msg_data);
	void setUUID(String uuid);
	void setFuncID(int id); // Switch among functions.
	void getOutputData(OutputData *data);
	void getApriltagExtraData(ApriltagExtraData *data);
	void getLanesExtraData(LanesExtraData *data);
	int getNumOfObjects();
	int getOutputFuncID();
	int getFuncID();
	String begin();
	String process();
	
	char* host = "192.168.11.11";
	char* target = "/";
	uint16_t port = 9030;
	   
private:
	void parseDetectionResult(DynamicJsonDocument json);
	String packFuncCMD();
	String packImgCMD();
	String packLoadFuncCMD();

	OutputData outputData_[MAX_OBJS];
	ApriltagExtraData apriltagData_[MAX_OBJS];
	LanesExtraData lanesData_;
	Harpcam::EState state_;
	String uuid_;
	bool detected_;
	int funcID_;
	int outputID_;
	int numObj_;
};

#endif
